package com.community.huiju.controller;


import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.common.base.BaseController;
import com.community.huiju.model.TpTransaction;
import com.community.huiju.service.ITransactionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpSession;

/**
 * <p>
 * 物业web端二手交易表 前端控制器
 * </p>
 *
 * @author jobob
 * @since 2018-12-29
 */
@RestController
@RefreshScope
@RequestMapping("/")
@Api(value = "二手租赁相关信息", description = "二手租赁相关信息Api")
public class TransactionController extends BaseController {
	
	@Autowired
	private ITransactionService transactionService;
	
	@ApiOperation(value = "根据搜索条件获取二手租赁列表", notes = "根据搜索条件获取二手租赁列表")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "query", dataType = "Integer", name = "transactionId", value = "帖子编号"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "transactionTitle", value = "帖子标题（模糊查询）"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "status", value = "发布状态"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "isReported", value = "是否被举报"),
			@ApiImplicitParam(paramType = "query", dataType = "integer", name = "pageNum", value = "分页第几页"),
			@ApiImplicitParam(paramType = "query", dataType = "integer", name = "pageSize", value = "分页每页长度")
	})
	@RequestMapping(value = "/transactions",method = RequestMethod.GET)
	public ResponseBean getList(@RequestParam(value = "transactionId", required = false) Integer transactionId,
	                            @RequestParam(value = "transactionTitle", required = false) String transactionTitle,
	                            @RequestParam(value = "status", required = false) String status,
	                            @RequestParam(value = "isReported", required = false) String isReported,
	                            @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
	                            @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize, HttpSession session){
		ResponseBean responseBean = new ResponseBean();
		UserElement userElement = getUserElement(session);
		TpTransaction tpTransaction = new TpTransaction();
		tpTransaction.setId(transactionId);
		tpTransaction.setTransactionTitle(transactionTitle);
		tpTransaction.setStatus(status);
		tpTransaction.setIsReported(isReported);
		tpTransaction.setCommunityId(userElement.getCommunityId());
		responseBean = transactionService.getTransactionList(tpTransaction,pageNum,pageSize);
		return responseBean;
	}
}
