package com.community.huiju.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.common.base.BaseController;
import com.community.huiju.model.TaUser;
import com.community.huiju.model.User;
import com.community.huiju.service.IUserService;
import com.community.huiju.vo.UserVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpSession;

/**
 * <p>
 * 物业web端用户表 前端控制器
 * </p>
 *
 * @author jobob
 * @since 2018-12-18
 */
@RestController
@RefreshScope
@RequestMapping("/")
@Api(value = "物业端用户API", description = "物业端用户API")
public class UserController extends BaseController {
	
	@Autowired
	private IUserService userService;

	
	@ApiOperation(value = "获取用户列表",nickname = "获取用户信息")
	@ApiImplicitParams({
			//@ApiImplicitParam(paramType = "header",dataType = "String",name = "X-Auth-Token",value = "Token"),
	})
	@RequestMapping(value = "/users",method = RequestMethod.GET)
	public ResponseBean getInfo(HttpSession session) {
		ResponseBean response = new ResponseBean();
		response.addSuccess(userService.list());
		Page a = new Page(1,10);
//		response.addSuccess(userService.page(a));
		return response;
	}

	@ApiOperation(value = "登陆", notes = "登陆")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "parameter", value = "loginName手机号;code验证码;communityId小区Id"),
			@ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 web"),
	})
	@RequestMapping(value = "/user/login", method = RequestMethod.POST)
	public ResponseBean login(@RequestBody String parameter, HttpSession session){
		ResponseBean responseBean = new ResponseBean();
		JSONObject jsonObject = JSONObject.parseObject(parameter);
		responseBean = userService.login(jsonObject.getString("loginName"),
				jsonObject.getInteger("communityId"),
				jsonObject.getString("code"));

		UserVO userVO = (UserVO)responseBean.getData();
		setUserElement(session,userVO);
		userVO.setToken(session.getId());

		return responseBean;
	}

	@ApiOperation(value = "获取用户信息", notes = "获取用户信息")
	@ApiImplicitParams({
		@ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "X-Auth-Token", value = "Token"),
		@ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 web"),
	})
	@RequestMapping(value = "/user/info", method = RequestMethod.GET)
	public ResponseBean info(HttpSession session){
		ResponseBean responseBean = new ResponseBean();
		UserElement userElement = getUserElement(session);
		responseBean = userService.userInfo(userElement);
		UserVO userVO = (UserVO)responseBean.getData();
		userVO.setCommunityId(userElement.getCommunityId());
		return responseBean;
	}

	@ApiOperation(value = "注销登录", notes = "注销登录")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "X-Auth-Token", value = "Token"),
			@ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 web"),
	})
	@RequestMapping(value = "/user/logOut", method = RequestMethod.POST)
	public ResponseBean longOut(HttpSession session){
		ResponseBean responseBean = new ResponseBean();
		cleanUserSession(session);
		responseBean.addSuccess("退出成功!");
		return responseBean;
	}
}
