package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.enums.SqlMethod;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.commom.utils.AccountValidatorUtil;
import com.community.commom.utils.BeanTools;

import com.community.huiju.dao.TaSysUserRoleMapper;
import com.community.huiju.dao.TaUserMapper;
import com.community.huiju.dao.UserMapper;
import com.community.huiju.exception.WisdomException;
import com.community.huiju.model.TaSysUserRole;
import com.community.huiju.model.TaUser;

import com.community.huiju.dao.TpBuildingOwnerInfoMapper;
import com.community.huiju.model.TpBuildingOwnerInfo;

import com.community.huiju.model.User;
import com.community.huiju.service.IBuildingOwnerInfoService;
import com.community.huiju.service.ITaUserService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.ibatis.binding.MapperMethod;
import org.apache.ibatis.session.SqlSession;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;


/**
 * <p>
 * 楼栋业主资料信息表 服务实现类
 * </p>
 *
 * @author weiximei
 * @since 2018-12-18
 */
@Service("iBuildingOwnerInfoService")
@Slf4j
public class BuildingOwnerInfoServiceImpl extends ServiceImpl<TpBuildingOwnerInfoMapper, TpBuildingOwnerInfo> implements IBuildingOwnerInfoService {

    @Autowired
    private TpBuildingOwnerInfoMapper tpBuildingOwnerInfoMapper;
    
    @Autowired
    private TaUserMapper taUserMapper;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private TaSysUserRoleMapper taSysUserRoleMapper;

    @Autowired
    private ITaUserService iTaUserService;
    
    public static final Logger logger = LoggerFactory.getLogger(BuildingOwnerInfoServiceImpl.class);
    
    @Override
    public ResponseBean listQuery(String parameter, UserElement userElement) {

        ResponseBean responseBean = new ResponseBean();

        JSONObject jsonObject = JSONObject.parseObject(parameter);
        TpBuildingOwnerInfo tpBuildingOwnerInfo = jsonObject.toJavaObject(TpBuildingOwnerInfo.class);

        Long pageNum = jsonObject.getLong("pageNum") == null ? 1 : jsonObject.getLong("pageNum");
        Long pageSize = jsonObject.getLong("pageSize") == null ? 10 : jsonObject.getLong("pageSize");

        Page<TpBuildingOwnerInfo> page = new Page(pageNum,pageSize);

        Map<String, Object> map = Maps.newHashMap();
        // 小区Id
        map.put("community_id", userElement.getCommunityId());
        // 栋
        map.put("building", tpBuildingOwnerInfo.getBuilding());
        // 单元
        map.put("unit", tpBuildingOwnerInfo.getUnit());
        // 楼层
        map.put("level", tpBuildingOwnerInfo.getLevel());
        // 房号
        map.put("room_no", tpBuildingOwnerInfo.getRoomNo());

        QueryWrapper<TpBuildingOwnerInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.allEq(map,false);
        queryWrapper.like(tpBuildingOwnerInfo.getOwnerName() != null,"owner_name", tpBuildingOwnerInfo.getOwnerName());

        // 分页查询
        IPage<TpBuildingOwnerInfo> infoIPage = tpBuildingOwnerInfoMapper.selectPage(page, queryWrapper);


        Map<String, Object> retMap = Maps.newHashMap();
        // 获取数据
        List<TpBuildingOwnerInfo> list = infoIPage.getRecords();

        retMap.put("list", list);
        retMap.put("total", infoIPage.getTotal());
        retMap.put("pageNum", infoIPage.getCurrent());
        retMap.put("pageSize", infoIPage.getSize());

        responseBean.addSuccess(retMap);

        return responseBean;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean update(String parameter, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();


        TpBuildingOwnerInfo tpBuildingOwnerInfo = JSONObject.parseObject(parameter, TpBuildingOwnerInfo.class);
        if (!AccountValidatorUtil.isPhone(tpBuildingOwnerInfo.getOwnerTel())){
            responseBean.addError("请输入正取的手机号!");
            return responseBean;
        }

        TpBuildingOwnerInfo oldTpBuildingOwnerInfo = tpBuildingOwnerInfoMapper.selectById(tpBuildingOwnerInfo.getId());
        if (null == oldTpBuildingOwnerInfo) {
            throw new WisdomException("%s 数据不存在!", tpBuildingOwnerInfo.getId());
        }
        if ("1".equals(oldTpBuildingOwnerInfo.getVerifyStatus())) {
            responseBean.addError("不能修改已认证的楼栋业主信息!");
            return responseBean;
        }

        // 物业端的用户,也就是正在操作的物业人员
        // 物业端用户小区Id不使用! 小区 Id 获取请使用 userElement
        User user = userMapper.selectById(userElement.getId());

        /**
         * 修改的时候, 如果提交上来的数据和要被修改的数据是一样的! 直接返回操作成功!
         */

        // 标记是否修改了手机号
        boolean phoneBool = false;

        // 校验楼栋信息表
        TpBuildingOwnerInfo selectBuild = getBuildingInfo(tpBuildingOwnerInfo, oldTpBuildingOwnerInfo.getCommunityId());
        // 如果数据查询到了, 并且手机号和传上来的手机号是一样的是, 代表就是原数据, 直接返回操作成功
        if (null != selectBuild && selectBuild.getOwnerTel().equals(tpBuildingOwnerInfo.getOwnerTel())) {
            tpBuildingOwnerInfoMapper.updateById(tpBuildingOwnerInfo);
            responseBean.addSuccess("操作成功!");
            return responseBean;

            // 如果 传上来的 楼栋,单元,楼层,单元,户号 查询出来的数据, 和 要修改的数据的id是匹配的, 就表示是只修改手机号
        } else if (null != selectBuild && (tpBuildingOwnerInfo.getId().intValue() == selectBuild.getId().intValue())) {
            // 此时要验证手机号是否存在楼栋资料库库里面
            // 校验楼栋信息表里面的业主手机号
            QueryWrapper<TpBuildingOwnerInfo> queryWrapperPhone = new QueryWrapper<>();
            queryWrapperPhone.eq("owner_tel", tpBuildingOwnerInfo.getOwnerTel());
            TpBuildingOwnerInfo selectBuildPhone = tpBuildingOwnerInfoMapper.selectOne(queryWrapperPhone);

            // 如果 通过手机号查询出来的楼栋信息数据的 id 和 传上来的 楼栋,单元,楼层,单元,户号 查询出来的数据的id 不匹配, 那么就表示这个手机号已经绑定了其他的楼栋信息
            if (null != selectBuildPhone && selectBuildPhone.getId().intValue() != selectBuild.getId().intValue()){
                responseBean.addError("该手机号已存在!");
                return responseBean;
            }

            // 开始修改手机号
            phoneBool = true;

            // 查询到了数据, 就表示新数据, 在库里面已经存在! 可能就是别业主的! 不能被修改为当前这个业主的!
        } else if (null != selectBuild && phoneBool == false){
            responseBean.addError("该户号已存在!");
            return responseBean;
        }

        // 开始更新楼栋资料
        BeanTools.copyProperties(tpBuildingOwnerInfo, oldTpBuildingOwnerInfo);

        oldTpBuildingOwnerInfo.setUpdateDate(LocalDateTime.now());
        oldTpBuildingOwnerInfo.setUpdateUser(userElement.getId());

        int row = tpBuildingOwnerInfoMapper.updateById(oldTpBuildingOwnerInfo);
        if (row <= 0) {
            log.error("修改操作! 修改楼栋信息表数据失败! 更新ID: ", oldTpBuildingOwnerInfo.getId());
            throw new WisdomException("操作失败!");
        }

        /**
         * APP 端这里, 如果楼栋资料库是已认证和已关联的! 那么就不允许修改手机号
         */

        // 开始匹配 APP 端用户表
        // 查询 APP 端业主
        TaUser appOwnerUser = getByIdAndCommunityId(tpBuildingOwnerInfo.getOwnerTel(), oldTpBuildingOwnerInfo.getCommunityId());
        if (null != appOwnerUser
                && !"1".equals(oldTpBuildingOwnerInfo.getVerifyStatus())
                && !"1".equals(oldTpBuildingOwnerInfo.getPairStatus())) {
            // 修改APP端信息
            appOwnerUser.setUserName(tpBuildingOwnerInfo.getOwnerName());
            appOwnerUser.setVerifyStatus("1");
            appOwnerUser.setBuildingOwnerInfoId(tpBuildingOwnerInfo.getId());
            row = taUserMapper.updateById(appOwnerUser);

            // 更新 楼栋业主信息表 的认证,关联状态
            // 认证状态  0 是未认证    1是认证通过   2是已作废
            oldTpBuildingOwnerInfo.setVerifyStatus("1");
            // 关联状态   0 未关联  1 是已关联
            oldTpBuildingOwnerInfo.setPairStatus("1");
            row = tpBuildingOwnerInfoMapper.updateById(oldTpBuildingOwnerInfo);

            // 更新APP端用户的角色身份为 业主
            updateAppRole(tpBuildingOwnerInfo.getCommunityId(), appOwnerUser.getId(), Constant.OWNER_ID);
        }



        if (row <= 0) {
            log.error("修改操作! 修改楼栋信息表过程中, 修改 APP 端失败信息失败!");
            throw new WisdomException("操作失败!");
        } else {
            responseBean.addSuccess("操作成功!");
        }
        return responseBean;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean add(String parameter, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();

        TpBuildingOwnerInfo tpBuildingOwnerInfo = JSONObject.parseObject(parameter, TpBuildingOwnerInfo.class);
        // 物业端的用户,也就是正在操作的物业人员
        User user = userMapper.selectById(userElement.getId());
        if (!AccountValidatorUtil.isPhone(tpBuildingOwnerInfo.getOwnerTel())){
            responseBean.addError("请输入正取的手机号!");
            return responseBean;
        }

        QueryWrapper<TpBuildingOwnerInfo> queryWrapperPhone = new QueryWrapper<>();
        // 校验楼栋信息表里面的业主手机号
        queryWrapperPhone.eq("owner_tel", tpBuildingOwnerInfo.getOwnerTel());
        TpBuildingOwnerInfo selectBuild = tpBuildingOwnerInfoMapper.selectOne(queryWrapperPhone);
        if (null != selectBuild) {
            responseBean.addError("该手机号已经录入!");
            return responseBean;
        }

        // 校验楼栋信息表
        selectBuild = getBuildingInfo(tpBuildingOwnerInfo, userElement.getCommunityId());
        if (null != selectBuild) {
            responseBean.addError("该楼层的户号,已存在!");
            return responseBean;
        }

        // 主键是自增的, 所以设置为 null
        tpBuildingOwnerInfo.setId(null);
        tpBuildingOwnerInfo.setUpdateUser(userElement.getId());
        tpBuildingOwnerInfo.setUpdateDate(LocalDateTime.now());
        tpBuildingOwnerInfo.setCreateDate(LocalDateTime.now());
        tpBuildingOwnerInfo.setCreateUser(userElement.getId());
        tpBuildingOwnerInfo.setVerifyStatus("0");

        // 添加的时候, 默认是当前操作的小区
        tpBuildingOwnerInfo.setCommunityId(userElement.getCommunityId());

        int row = tpBuildingOwnerInfoMapper.insert(tpBuildingOwnerInfo);
        if (row <= 0) {
            log.error("添加操作! 楼栋业主信息表 失败!");
            throw new WisdomException("操作失败!");
        }

        // 开始匹配 APP 端用户表
        // 查询 APP 端业主
        TaUser appOwnerUser = getByIdAndCommunityId(tpBuildingOwnerInfo.getOwnerTel(), tpBuildingOwnerInfo.getCommunityId());
        if (null != appOwnerUser) {
            // 更新 APP 端用户认证的状态
            appOwnerUser.setBuildingOwnerInfoId(tpBuildingOwnerInfo.getId());
            appOwnerUser.setUserName(tpBuildingOwnerInfo.getOwnerName());

            // 认证状态  0 是未认证    1是认证通过   2是已作废
            // 标记位已认证
            appOwnerUser.setVerifyStatus("1");
            row = taUserMapper.updateById(appOwnerUser);
            if (row <= 0) {
                log.error("添加操作! 楼栋业主信息表 过程中 修改 APP 端用户认证的状态失败! ");
                throw new WisdomException("操作失败!");
            }

            // 更新 楼栋业主信息表 的认证,关联状态
            // 认证状态  0 是未认证    1是认证通过   2是已作废
            tpBuildingOwnerInfo.setVerifyStatus("1");
            // 关联状态   0 未关联  1 是已关联
            tpBuildingOwnerInfo.setPairStatus("1");
            row = tpBuildingOwnerInfoMapper.updateById(tpBuildingOwnerInfo);
            if (row <= 0) {
                log.error("添加操作! 更新 楼栋业主信息表 的认证,关联状态 失败!");
                throw new WisdomException("操作失败!");
            }

            // 更新APP端用户的角色身份为 业主
            updateAppRole(tpBuildingOwnerInfo.getCommunityId(), appOwnerUser.getId(), Constant.OWNER_ID);
        }


        if (row <= 0) {
            throw new WisdomException("操作失败!");
        } else {
            responseBean.addSuccess("操作成功!");
        }

        return responseBean;
    }

    /**
     * 修改 APP 端用户的角色
     * @param communityId 小区Id
     * @param appUserId APP 端用户Id
     * @param roleId 角色Id
     */
    private void updateAppRole(Integer communityId, Integer appUserId, Integer roleId) {
        // 更新APP端用户的角色身份
        taSysUserRoleMapper.deleteByCommunityIdAndUserId(communityId, appUserId);
        TaSysUserRole taSysUserRole = new TaSysUserRole();
        taSysUserRole.setUserId(appUserId);
        taSysUserRole.setRoleId(Constant.OWNER_ID);
        taSysUserRole.setCommunityId(communityId);
        taSysUserRoleMapper.insert(taSysUserRole);
    }

    @Override
    public ResponseBean getBuildingOrUnitOrNumber(String parameter,Integer communityId) {

        ResponseBean responseBean = new ResponseBean();
        if (null == communityId || "".equals(communityId)) {
            throw new WisdomException("编号:%s 小区不存在!", communityId);
        }

        TpBuildingOwnerInfo tpBuildingOwnerInfo = JSONObject.parseObject(parameter, TpBuildingOwnerInfo.class);

        Map<String, Object> map = Maps.newHashMap();

        // 小区
        map.put("community_id", communityId);
        // 栋
        map.put("building", tpBuildingOwnerInfo.getBuilding());
        // 单元
        map.put("unit", tpBuildingOwnerInfo.getUnit());
        // 楼层
        map.put("level", tpBuildingOwnerInfo.getLevel());

        QueryWrapper<TpBuildingOwnerInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.allEq(map, false);

        /**
         * 通过分组来去重
         *
         * 1. building 为空表示为 building 字段进行分组
         * 2. unit 为空表示为 unit 字段进行分组
         * 3. level 为空表示为 level 字段进行分组
         * 4. room_no 为空表示为 room_no 字段进行分组
         */
        if (null == tpBuildingOwnerInfo.getBuilding() || "".equals(tpBuildingOwnerInfo.getBuilding())) {
            queryWrapper.groupBy("building");
        } else if (null == tpBuildingOwnerInfo.getUnit() || "".equals(tpBuildingOwnerInfo.getUnit())) {
            queryWrapper.groupBy("unit");
        } else if (null == tpBuildingOwnerInfo.getLevel() || "".equals(tpBuildingOwnerInfo.getLevel())) {
            queryWrapper.groupBy("level");
        } else if (null == tpBuildingOwnerInfo.getRoomNo() || "".equals(tpBuildingOwnerInfo.getRoomNo())) {
            queryWrapper.groupBy("room_no");
        }

        /**
         * 1.如果 楼栋值不存在表示根据小区Id查询时查询楼栋!
         * 2.如果 楼栋存在表示查询单元
         * 3.如果 单元存在表示查询楼层
         * 4.如果 楼层存在表示查询户号
         *
         * 查询户号, 依赖于前面的查询条件成立!
         *
         * 这里的查询都依赖于, 前面的查询条件成立! 比如 要使用条件2, 那么条件1 必须成立!
         *
         */

        List<TpBuildingOwnerInfo> selectList = tpBuildingOwnerInfoMapper.selectList(queryWrapper);

        responseBean.addSuccess(selectList);

        return responseBean;
    }
    
    /**
     * 上传excel里面的内容
     *
     * @param file
     * @param userElement
     * @return
     */
    @Override
    public ResponseBean submitExcelData(MultipartFile file, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = getExcelData(file);
        if (!responseBean.getCode().equals(Constant.RESPONSE_SUCCESS)){
            return responseBean;
        }
        //获取要上传的数据
        Map<String,Object> map = (Map<String, Object>) responseBean.getData();
        List<TpBuildingOwnerInfo> list = (List<TpBuildingOwnerInfo>) map.get("list");
        list.stream().forEach(BuildingOwnerInfo -> {
            BuildingOwnerInfo.setCommunityId(userElement.getCommunityId());
            BuildingOwnerInfo.setVerifyStatus(Constant.UN_VERIFY_STATUS);
            BuildingOwnerInfo.setCreateUser(userElement.getId());
            BuildingOwnerInfo.setCreateDate(LocalDateTime.now());
            BuildingOwnerInfo.setUpdateUser(userElement.getId());
            BuildingOwnerInfo.setUpdateDate(LocalDateTime.now());
        });
        //批量插入
	    tpBuildingOwnerInfoMapper.batchInsert(list);
        //判断是否认证过
        List<TpBuildingOwnerInfo> updateList = new ArrayList<TpBuildingOwnerInfo>();
        List<TaUser> updataTaUserList = new ArrayList<TaUser>();
        list.stream().forEach(BuildingOwnerInfo -> {
            TaUser taUser = taUserMapper.selectByTel(BuildingOwnerInfo.getOwnerTel(),BuildingOwnerInfo.getCommunityId());
            if (null != taUser){
                BuildingOwnerInfo.setVerifyStatus(Constant.VERIFY_STATUS);
                updateList.add(BuildingOwnerInfo);
                taUser.setVerifyStatus(Constant.VERIFY_STATUS);
                taUser.setBuildingOwnerInfoId(BuildingOwnerInfo.getId());
                updataTaUserList.add(taUser);
            }
        });
        //批量更新
        if (updateList.size() > 0){
	        tpBuildingOwnerInfoMapper.batchUpdate(updateList);
        }
        if (updataTaUserList.size() > 0){
            taUserMapper.batchUpdate(updataTaUserList);
            taUserMapper.batchUpdateRole(updataTaUserList);
        }
        responseBean.addSuccess("success");
        return responseBean;
    }

    @Override
    public ResponseBean updataApprove(String paramets) {
        ResponseBean response = new ResponseBean();
        JSONObject jsonobject = JSONObject.parseObject(paramets);
        String ownerTel=jsonobject.getString("ownerTel");
        String verifyStatus=jsonobject.getString("verifyStatus");
        TaUser user = new TaUser();
        //用手机号去查当前用户id
        Integer userId = taUserMapper.getUserID(ownerTel);
        user.setId(userId);
        int vs = Integer.parseInt(verifyStatus);
        if (1==vs) {
            //停用当前业主认证状态，业主和家属将租客将无法使用
//            user.setVerifyStatus("2");
        }else {
        //启动当前业主，判断业主下是否有关联业主或则家属，如果也关联家属返回有关联的用户，如果没有将可以修改认证状态
        List<TaUser> userParent=taUserMapper.selectUser(userId);
        for(TaUser users:userParent){
            if(null !=users.getParentId()&&userId.equals(users.getParentId())){
                Map<String, Object> map = Maps.newHashMap();
                map.put("code","0");
                map.put("data","有关联的用户");
                response.addSuccess(map);
                return response;
            }
        }
//            user.setVerifyStatus("1");
        }
//        taUserMapper.updateByPrimaryKeySelective(user);
        return response;
    }

    /**
     * 根据excel获取相关信息
     *
     * @param file
     * @return
     */
    @Override
    public ResponseBean getExcelData(MultipartFile file) {
        ResponseBean responseBean = new ResponseBean();
        List<TpBuildingOwnerInfo> list = new ArrayList<>();
        if (file == null) {
            responseBean.addError("对象不能为空");
            return responseBean;
        }
        //获取文件的名字
        String originalFilename = file.getOriginalFilename();
        Workbook workbook = null;
        try {
            if (originalFilename.endsWith(Constant.SUFFIX_2003)) {
                workbook = new HSSFWorkbook(file.getInputStream());
            } else if (originalFilename.endsWith(Constant.SUFFIX_2007)) {
                workbook = new XSSFWorkbook(file.getInputStream());
            }
        } catch (Exception e) {
            logger.info(originalFilename);
            e.printStackTrace();
            responseBean.addError("格式错误");
            return responseBean;
        }
        if (workbook == null) {
            logger.info(originalFilename);
            responseBean.addError("格式错误");
            return responseBean;
        } else {
            List<String> temBuildingList = new ArrayList<String>();
            List<String> temTelList = new ArrayList<String>();
            //获取一个sheet也就是一个工作簿
            Sheet sheet = workbook.getSheetAt(0);
            int lastRowNum = sheet.getLastRowNum();
            //从第一行开始第一行一般是标题
            for (int j = 3; j <= lastRowNum; j++) {
                Row row = sheet.getRow(j);
                TpBuildingOwnerInfo tpBuildingOwnerInfo = new TpBuildingOwnerInfo();
                String phase = row.getCell(0).getStringCellValue().trim();
                String building = row.getCell(1).getStringCellValue().trim();
                String unit = row.getCell(2).getStringCellValue().trim();
                String level = row.getCell(3).getStringCellValue().trim();
                String roomNo = row.getCell(4).getStringCellValue().trim();
                String ownerName = row.getCell(5).getStringCellValue().trim();
                String ownerTel = row.getCell(6).getStringCellValue().trim();
                int currentRow = j+1;
                if (StringUtils.isEmpty(phase)){
                    responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
                    return responseBean;
                }
                if (StringUtils.isEmpty(building)){
                    responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
                    return responseBean;
                }
                if (StringUtils.isEmpty(unit)){
                    responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
                    return responseBean;
                }
                if (StringUtils.isEmpty(level)){
                    responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
                    return responseBean;
                }
                if (StringUtils.isEmpty(roomNo)){
                    responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
                    return responseBean;
                }
                if (StringUtils.isEmpty(ownerName)){
                    responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
                    return responseBean;
                }
                if (StringUtils.isEmpty(ownerTel)){
                    responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
                    return responseBean;
                }
                //校验手机号码
                if (!AccountValidatorUtil.isPhone(ownerTel)){
                    responseBean.addError("请输入正确手机号:" + ownerTel);
                    return responseBean;
                }
                String buildingInfo = phase + building + unit + level + roomNo;
                if (temBuildingList.contains(buildingInfo)){
                    logger.info("存在重复房源:{}",buildingInfo);
                    responseBean.addError("存在重复房源:"+ buildingInfo);
                    return responseBean;
                }
                if (temTelList.contains(ownerTel)){
                    logger.info("存在重复手机号:{}",ownerTel);
                    responseBean.addError("存在重复手机号:"+ ownerTel);
                    return responseBean;
                }
                //把数据塞入temList校验是否有重复的数据
                temBuildingList.add(buildingInfo);
                temTelList.add(ownerTel);
                //构建数据
                tpBuildingOwnerInfo.setPhase(phase);
                tpBuildingOwnerInfo.setBuilding(building);
                tpBuildingOwnerInfo.setUnit(unit);
                tpBuildingOwnerInfo.setLevel(level);
                tpBuildingOwnerInfo.setRoomNo(roomNo);
                tpBuildingOwnerInfo.setOwnerName(ownerName);
                tpBuildingOwnerInfo.setOwnerTel(ownerTel);
                list.add(tpBuildingOwnerInfo);
            }
        }
        //构建分页
        Map<String,Object> data = new HashMap<String, Object>();
        data.put("list",list);
        data.put("total",list.size());
        
        responseBean.addSuccess(data);
        return responseBean;
    }
    //判断整数
    public static boolean isNumeric(String str){
    for (int i = str.length();--i>=0;){
        if (!Character.isDigit(str.charAt(i))){
            return false;
        }}
            return true;
        }

    @Override
    public ResponseBean selectUserApprove(String paramets,Integer CommunityId) {

        ResponseBean responseBean = new ResponseBean();

        JSONObject jsonobject = JSONObject.parseObject(paramets);
        Integer pageNum = jsonobject.getInteger("pageNum");
        Integer pageSize = jsonobject.getInteger("pageSize");
        TpBuildingOwnerInfo tpBuildingOwnerInfo = JSONObject.parseObject(paramets, TpBuildingOwnerInfo.class);

        // 分页
        Page<TpBuildingOwnerInfo> page = new Page<>();
        page.setSize(pageSize == null ? 10 : pageSize);
        page.setCurrent(pageNum == null ? 1 : pageNum);

        IPage<TpBuildingOwnerInfo> ownerInfoIPage = tpBuildingOwnerInfoMapper.selectUserApprove(page, tpBuildingOwnerInfo.getOwnerName(),tpBuildingOwnerInfo.getOwnerTel(),tpBuildingOwnerInfo.getVerifyStatus(),CommunityId);
        List<TpBuildingOwnerInfo> infoList = ownerInfoIPage.getRecords();
        infoList.stream().forEach(e->{
            if (null!=e.getBuilding()&&!"".equals(e.getBuilding())&&isNumeric(e.getBuilding())){
                e.setBuilding(e.getBuilding().length()+"栋");
            }
            if (null!=e.getUnit()&&!"".equals(e.getUnit())&&isNumeric(e.getUnit())){
                e.setUnit(e.getUnit()+"单元");
            }
            if (null!=e.getLevel()&&!"".equals(e.getLevel())&&isNumeric(e.getLevel())){
                e.setLevel(e.getLevel()+"层");
            }
            if (null!=e.getRoomNo()&&!"".equals(e.getRoomNo())&&isNumeric(e.getRoomNo())){
                e.setRoomNo(e.getRoomNo()+"户");
            }
            if (null!=e.getPhase()&&!"".equals(e.getPhase())&&isNumeric(e.getPhase())){
                e.setPhase(e.getPhase()+"期");
            }
        });

        Map<String, Object> map = Maps.newHashMap();
        map.put("list", infoList);
        map.put("total", ownerInfoIPage.getTotal());
        map.put("pageNum", ownerInfoIPage.getCurrent());
        map.put("pageSize", ownerInfoIPage.getSize());

        responseBean.addSuccess(map);

        return responseBean;
    }

//    @Override
//    @Transactional(rollbackFor = Exception.class)
//    @Deprecated
//    public ResponseBean deleteIds(List<Integer> ids, Integer propertyUserId) {
//
//        ResponseBean responseBean = new ResponseBean();
//
//        List<TpBuildingOwnerInfo> infoList = ids.stream().map(e->{
//            TpBuildingOwnerInfo tpBuildingOwnerInfo = new TpBuildingOwnerInfo();
//            tpBuildingOwnerInfo.setId(e);
//            tpBuildingOwnerInfo.setVerifyStatus("2");
//            tpBuildingOwnerInfo.setUpdateUser(propertyUserId);
//            tpBuildingOwnerInfo.setUpdateDate(LocalDateTime.now());
//            return tpBuildingOwnerInfo;
//        }).collect(Collectors.toList());
//        boolean result = updateBatchById(infoList);
//
//        // APP 端用户
//        List<TaUser> taUserList = Lists.newArrayList();
//
//        ids.forEach(e-> {
//            TpBuildingOwnerInfo tpBuildingOwnerInfo = tpBuildingOwnerInfoMapper.selectById(e);
//            if (null != tpBuildingOwnerInfo) {
//                // 查询 APP 端业主
//                TaUser appOwnerUser = getByIdAndCommunityId(tpBuildingOwnerInfo.getOwnerTel(), tpBuildingOwnerInfo.getCommunityId());
//                if (null != appOwnerUser) {
//                    appOwnerUser.setVerifyStatus(2+"");
//                    appOwnerUser.setUpdateDate(LocalDateTime.now());
//                    taUserList.add(appOwnerUser);
//                }
//
//            }
//        });
//
//        if (null != taUserList && taUserList.size() > 0) {
//            result = iTaUserService.updateBatchById(taUserList);
//        }
//
//
//        if (result) {
//            responseBean.addSuccess("操作成功!");
//        }else {
//            responseBean.addError("操作失败!");
//        }
//
//
//        return responseBean;
//    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean deleteIds(List<Integer> ids, Integer propertyUserId) {

        ResponseBean responseBean = new ResponseBean();
        ids.stream().forEach(e->{
            TpBuildingOwnerInfo buildingOwnerInfo = tpBuildingOwnerInfoMapper.selectById(e);
            if ("1".equals(buildingOwnerInfo.getVerifyStatus())) {
                throw new WisdomException("不能删除已认证的楼栋资料!");
                /**
                 * 批量删除的时候,只要存在一个已认证的楼栋资料, 就停止删除!
                 */
            }
            tpBuildingOwnerInfoMapper.deleteById(buildingOwnerInfo.getId());
        });

        responseBean.addSuccess("操作成功!");
        return responseBean;
    }

    /**
     * 根据 手机号 和 小区Id 查询用户
     * @param phone
     * @param communityId
     * @return
     */
    private TaUser getByIdAndCommunityId(String phone, Integer communityId){
        // 开始匹配 APP 端用户表
        QueryWrapper<TaUser> taUserQueryWrapper = new QueryWrapper<>();
        taUserQueryWrapper.eq("login_name", phone);
        taUserQueryWrapper.eq("community_id", communityId);
        // 查询 APP 端业主
        TaUser appOwnerUser = taUserMapper.selectOne(taUserQueryWrapper);

        return appOwnerUser;
    }

    /**
     * 查询 楼栋信息
     *      根据小区Id, 栋,单元,楼层,房号
     * @param tpBuildingOwnerInfo 楼栋信息
     * @param communityId 小区Id
     * @return
     */
    private TpBuildingOwnerInfo getBuildingInfo(TpBuildingOwnerInfo tpBuildingOwnerInfo, Integer communityId) {
        // 校验楼栋信息表
        QueryWrapper<TpBuildingOwnerInfo> queryWrapper = new QueryWrapper<>();
        // 小区ID
        queryWrapper.eq("community_id", communityId);
        // 栋
        queryWrapper.eq("building", tpBuildingOwnerInfo.getBuilding());
        // 单元
        queryWrapper.eq("unit", tpBuildingOwnerInfo.getUnit());
        // 楼层
        queryWrapper.eq("level", tpBuildingOwnerInfo.getLevel());
        // 房号
        queryWrapper.eq("room_no", tpBuildingOwnerInfo.getRoomNo());
        TpBuildingOwnerInfo selectBuild = tpBuildingOwnerInfoMapper.selectOne(queryWrapper);

        return selectBuild;
    }
}
