package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.commom.utils.BeanTools;
import com.community.huiju.dao.TdImagesMapper;
import com.community.huiju.dao.TpActivityMapper;
import com.community.huiju.exception.WisdomException;
import com.community.huiju.model.TdImages;
import com.community.huiju.model.TpActivity;
import com.community.huiju.model.User;
import com.community.huiju.service.ITdImagesService;
import com.community.huiju.service.ITpActivityService;
import com.community.huiju.service.IUserService;
import com.community.huiju.vo.ActivityVO;
import com.google.common.collect.Maps;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 物业web端活动表 服务实现类
 * </p>
 *
 * @author weiximei
 * @since 2018-12-28
 */
@Service
public class TpActivityServiceImpl extends ServiceImpl<TpActivityMapper, TpActivity> implements ITpActivityService {

    @Autowired
    private TpActivityMapper tpActivityMapper;

    @Autowired
    private TdImagesMapper tdImagesMapper;

    @Autowired
    private ITdImagesService iTdImagesService;

    // 物业用户
    @Autowired
    private IUserService iUserService;

    @Override
    public ResponseBean listQuery(String parameter, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();

        JSONObject jsonObject = JSONObject.parseObject(parameter);
        // TpActivity tpActivity = jsonObject.toJavaObject(TpActivity.class);
        String id = jsonObject.getString("id");
        String activityTitle = jsonObject.getString("activityTitle");
        Long pageNum = jsonObject.getLong("pageNum") == null ? 1 : jsonObject.getLong("pageNum");
        Long pageSize = jsonObject.getLong("pageSize") == null ? 10 : jsonObject.getLong("pageSize");

        Page<TpActivity> page = new Page<>();
        page.setCurrent(pageNum);
        page.setSize(pageSize);

        QueryWrapper<TpActivity> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(id != null && !"".equals(id),"id", id);
        queryWrapper.like(activityTitle != null && !"".equals(activityTitle),"activity_title", activityTitle);
        queryWrapper.eq("community_id", userElement.getCommunityId());
        queryWrapper.orderByDesc("sort","create_date");

        IPage<TpActivity> infoPage = tpActivityMapper.selectPage(page, queryWrapper);
        List<TpActivity> activityList = infoPage.getRecords();
        List<ActivityVO> activityVOList = activityList.stream().map(activity-> {
            ActivityVO activityVO = new ActivityVO();
            BeanUtils.copyProperties(activity, activityVO);
            User createUser = iUserService.getById(activity.getCreateUser());
            User updateUser = iUserService.getById(activity.getUpdateUser());
            activityVO.setCrateUserName(createUser.getUserName());
            activityVO.setUpdateUserName(updateUser.getUserName());
            return activityVO;
        }).collect(Collectors.toList());

        Map<String, Object> map = Maps.newHashMap();
        map.put("list", activityVOList);
        map.put("total", infoPage.getTotal());
        map.put("pageNum", infoPage.getCurrent());
        map.put("pageSize", infoPage.getSize());

        responseBean.addSuccess(map);

        return responseBean;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean addActivity(String parameter, UserElement userElement) {

        ResponseBean responseBean = new ResponseBean();
        JSONObject jsonObject = JSONObject.parseObject(parameter);
        // 活动配图
        JSONArray jsonArray = jsonObject.getJSONArray("contentImg");
        String [] contentImgArray = jsonArray.toArray(new String[]{});
        TpActivity tpActivity = jsonObject.toJavaObject(TpActivity.class);

        if (contentImgArray.length <= 0 && (null == tpActivity.getActivityContent() || "".equals(tpActivity.getActivityContent()))){
            responseBean.addError("活动内容描述 和 活动配图至少要存在一个!");
            return responseBean;
        }

        tpActivity.setCommunityId(userElement.getCommunityId());
        tpActivity.setUpdateDate(LocalDateTime.now());
        tpActivity.setUpdateUser(userElement.getId());
        tpActivity.setCreateDate(LocalDateTime.now());
        tpActivity.setCreateUser(userElement.getId());

        int row = tpActivityMapper.insert(tpActivity);

        // 插入图片
        insertImg(contentImgArray, userElement, tpActivity);

        if (row <= 0) {
            throw new WisdomException("操作失败!");
        } else {
            responseBean.addSuccess("操作成功!");
        }

        return responseBean;
    }

    /**
     * 插入图片
     * @param contentImgArray
     * @param userElement
     * @param tpActivity
     */
    private void insertImg(String [] contentImgArray, UserElement userElement, TpActivity tpActivity) {
        if (contentImgArray == null) {
            return;
        }
        for (String img : contentImgArray) {
            TdImages tdImages = new TdImages();
            tdImages.setCreateTime(LocalDateTime.now());
            tdImages.setCreateUser(userElement.getId());
            tdImages.setImageUrl(img);
            tdImages.setType("activity");
            tdImages.setUuid(tpActivity.getId());

            tdImagesMapper.insert(tdImages);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean updateActivity(String parameter, UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();
        JSONObject jsonObject = JSONObject.parseObject(parameter);
        TpActivity tpActivity = jsonObject.toJavaObject(TpActivity.class);

        // 活动配图
        JSONArray jsonArray = jsonObject.getJSONArray("contentImg");
        String [] contentImgArray = jsonArray.toArray(new String[]{});

        TpActivity selectTpActivity = tpActivityMapper.selectById(tpActivity.getId());
        if (null == selectTpActivity) {
            responseBean.addError("活动不存在!");
            return responseBean;
        }

        BeanTools.copyProperties(tpActivity, selectTpActivity);

        selectTpActivity.setUpdateDate(LocalDateTime.now());
        selectTpActivity.setUpdateUser(userElement.getId());

        int row = tpActivityMapper.updateById(selectTpActivity);

        // 先将原图删除!
        if (contentImgArray.length > 0) {
            QueryWrapper<TdImages> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("uuid", tpActivity.getId());
            queryWrapper.eq("type", "activity");
            tdImagesMapper.delete(queryWrapper);
        }

        // 插入图片
        insertImg(contentImgArray, userElement, tpActivity);

        if (row <= 0) {
            throw new WisdomException("操作失败!");
        } else {
            responseBean.addSuccess("操作成功!");
        }

        return responseBean;
    }

    @Override
    public ResponseBean getById(Integer id) {
        ResponseBean responseBean = new ResponseBean();

        TpActivity tpActivity = tpActivityMapper.selectById(id);

        QueryWrapper<TdImages> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("uuid", tpActivity.getId());
        queryWrapper.eq("type", "activity");
        List<TdImages> imagesList = tdImagesMapper.selectList(queryWrapper);

        // 转换, 只保留 imageUrl 字段
        List<String> stringList = imagesList.stream().map(e->new String(e.getImageUrl())).collect(Collectors.toList());

        Map<String, Object> map = Maps.newHashMap();
        map.put("info", tpActivity);
        map.put("contentImg", stringList);

        responseBean.addSuccess(map);

        return responseBean;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean updateStatusInvalid(List<Integer> ids, UserElement userElement) {

        ResponseBean responseBean = new ResponseBean();
        List<TpActivity> tpActivityList = (List<TpActivity>) listByIds(ids);
        tpActivityList.stream().forEach(e-> {
            e.setStatus("0");
            e.setUpdateUser(userElement.getId());
        });
        updateBatchById(tpActivityList);

        responseBean.addSuccess("操作成功!");

        return responseBean;
    }
}
