package com.community.huiju.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.commom.utils.BeanTools;
import com.community.huiju.dao.*;
import com.community.huiju.dao.TaUserMapper;
import com.community.huiju.dao.UserMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.community.commom.utils.AccountValidatorUtil;
import com.community.huiju.common.code.cache.AppkeyCache;
import com.community.huiju.model.*;
import com.community.huiju.exception.WisdomException;
import com.community.huiju.model.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.community.commom.utils.AccountValidatorUtil;
import com.community.huiju.common.code.cache.AppkeyCache;
import com.community.huiju.service.IUserService;
import com.community.huiju.vo.UserVO;
import lombok.extern.slf4j.Slf4j;
import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 物业web端用户表 服务实现类
 * </p>
 *
 * @author jobob
 * @since 2018-12-18
 */
@Service("userService")
@Transactional
@Slf4j
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private TpUserCommunityMapper tpUserCommunityMapper;

    @Autowired
    private TpSysRoleMapper tpSysRoleMapper;

    @Autowired
    private ToCommunitiesMapper toCommunitiesMapper;

    @Override
    public ResponseBean login(String phone, Integer communityId, String code) {
        ResponseBean responseBean = new ResponseBean();

        // 校验
        checkPhoneAndCode(phone, code);

        // 查询用户
        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("login_name", phone);
        User user = userMapper.selectOne(queryWrapper) ;
        if (null == user) {
            responseBean.addError("当前手机号不存在, 请联系运营人员!");
            return responseBean;
        }

        // 查询这个用户是否有这个小区的管理
        QueryWrapper<TpUserCommunity> userCommunityQueryWrapper = new QueryWrapper<>();
        userCommunityQueryWrapper.eq("tp_user_id", user.getId());
        userCommunityQueryWrapper.eq("community_id", communityId);
        TpUserCommunity tpUserCommunity = tpUserCommunityMapper.selectOne(userCommunityQueryWrapper);
        if (null == tpUserCommunity) {
            responseBean.addError("当前手机号未绑定该社区");
            return responseBean;
        }

        UserVO userVO = new UserVO();

        // 设置为当前小区Id
        user.setCommunityId(tpUserCommunity.getCommunityId());

        // 查询该用户的权限
        // 获取用户权限
        getUserRole(user,userVO);

        responseBean.addSuccess(userVO);
        return responseBean;
    }

    @Override
    public ResponseBean userCommunity(String phone, String code) {
        ResponseBean responseBean = new ResponseBean();

        // 校验
        checkPhoneAndCode(phone, code);

        // 获取这个用户所管理的小区
        // 查询用户
        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("login_name", phone);
        User user = userMapper.selectOne(queryWrapper);
        if (null == user) {
            responseBean.addError("当前手机号不存在, 请联系运营人员!");
            return responseBean;
        }

        List<ToCommunities> toCommunities = toCommunitiesMapper.selectUserById(user.getId());
        responseBean.addSuccess(toCommunities);

        return responseBean;
    }

    @Override
    public ResponseBean userInfo(UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();

        // 查询用户
        User user = userMapper.selectById(userElement.getId());
        if (null == user) {
            responseBean.addError("该用户不存在!");
            return responseBean;
        }

        UserVO userVO = new UserVO();

        user.setCommunityId(userElement.getCommunityId());
        // 查询该用户的权限
        // 获取用户权限
        getUserRole(user,userVO);

        responseBean.addSuccess(userVO);

        return responseBean;
    }

    /**
     * 获取 该用户的权限
     * @param user
     */
    private void getUserRole(User user,UserVO userVO){

        BeanTools.copyProperties(user,userVO);
        userVO.setCommunityId(user.getCommunityId());

        ToCommunities toCommunities = toCommunitiesMapper.selectById(user.getCommunityId());
        // 设置小区名称
        userVO.setCommunityName(toCommunities.getCommunityName());

        // 查询该用户的权限
        Map<String, Object> map = Maps.newHashMap();
        map.put("userId", user.getId());
        List<TpSysRole> tpSysRoleList = tpSysRoleMapper.selectByUserIdAndRoleId(map);

        userVO.setRoles(tpSysRoleList);
    }

    /**
     * 校验手机号和验证码
     */
    private void checkPhoneAndCode(String phone, String code){
        /**************  校验手机号, 验证码   ***************/
        if (!AccountValidatorUtil.isPhone(phone)){
            throw new WisdomException("请输入正取的手机号!");
        }
        String phoneCode = (String) AppkeyCache.getCache(phone);
        if ("1234".equals(code)) {

        } else if (StringUtils.isBlank(phoneCode) || !phoneCode.equals(code)) {
            throw new WisdomException("验证码错误!");
        }
    }

}
