import Vue from 'vue'
import Router from 'vue-router'

// in development-env not use lazy-loading, because lazy-loading too many pages will cause webpack hot update too slow. so only in production use lazy-loading;
// detail: https://panjiachen.github.io/vue-element-admin-site/#/lazy-loading

Vue.use(Router)

/* Layout */
import Layout from '../views/layout/Layout'

/**
* hidden: true                   if `hidden:true` will not show in the sidebar(default is false)
* alwaysShow: true               if set true, will always show the root menu, whatever its child routes length
*                                if not set alwaysShow, only more than one route under the children
*                                it will becomes nested mode, otherwise not show the root menu
* redirect: noredirect           if `redirect:noredirect` will no redirect in the breadcrumb
* name:'router-name'             the name is used by <keep-alive> (must set!!!)
* meta : {
    title: 'title'               the name show in submenu and breadcrumb (recommend set)
    icon: 'svg-name'             the icon show in the sidebar,
  }
**/
export const constantRouterMap = [
  { path: '/login', component: () => import('@/views/login/index'), hidden: true },
  { path: '/404', component: () => import('@/views/404'), hidden: true },
  {
    path: '/dashboard',
    component: Layout,
    redirect: '/dashboard',
    name: 'Dashboard',
    hidden: true,
    children: [{
      path: 'dashboard',
      component: () => import('@/views/dashboard/index')
    }]
  },

  {
    path: '/',
    component: Layout,
    redirect: '/index',
    children: [
      {
        path: '',
        component: () => import('@/views/index/index'),
        name: 'index',
        meta: { title: '首页', icon: 'index' }
      }
    ]
  },
  {
    path: '/building',
    component: Layout,
    redirect: '/building/index',
    alwaysShow: true,
    meta: { title: '业主/楼栋', icon: 'zip' },
    children: [
      {
        path: '/building/index',
        component: () => import('@/views/building/index'),
        name: 'building-index',
        meta: { title: '楼栋业主资料库', icon: 'building' }
      },
      {
        path: '/owner/index',
        component: () => import('@/views/owner/index'),
        name: 'owner-index',
        meta: { title: '已认证业主', icon: 'building' }
      },
      {
        path: '/building/batch',
        component: () => import('@/views/building/batch/batchImport'),
        name: 'batch-import',
        hidden: true,
        meta: { title: '批量导入楼栋信息', icon: 'table' }
      },
      {
        path: '/building/add',
        component: () => import('@/views/building/add/index'),
        name: 'building-add',
        hidden: true,
        meta: { title: '添加楼栋业主资料', icon: 'table' }
      },
      {
        path: '/building/edi',
        component: () => import('@/views/building/edi/index'),
        name: 'building-edi',
        hidden: true,
        meta: { title: '修改楼栋业主资料', icon: 'table' }
      }
    ]
  },
  {
    path: '/social',
    component: Layout,
    redirect: '/social/index',
    alwaysShow: true,
    meta: { title: '论坛内容', icon: 'zip' },
    children: [
      {
        path: '/social/announcement',
        component: () => import('@/views/social/announcement/index'),
        name: 'announcement-index',
        meta: { title: '公告内容', icon: 'table' }
      },
      {
        path: '/social/activity',
        component: () => import('@/views/social/activity/index'),
        name: 'activity-index',
        meta: { title: '活动管理', icon: 'table' }
      },
      {
        path: '/social/transaction',
        component: () => import('@/views/social/transaction/index'),
        name: 'transaction-import',
        meta: { title: '二手求购租赁管理', icon: 'table' }
      },
      {
        path: '/activity/add',
        component: () => import('@/views/social/activity/add/index'),
        name: 'activity-add',
        hidden: true,
        meta: { title: '添加活动', icon: 'table' }
      },
      {
        path: '/activity/edi',
        component: () => import('@/views/social/activity/edi/index'),
        name: 'activity-edi',
        hidden: true,
        meta: { title: '修改活动', icon: 'table' }
      },
      {
        path: '/activity/info',
        component: () => import('@/views/social/activity/info/index'),
        name: 'activity-info',
        hidden: true,
        meta: { title: '活动详情', icon: 'table' }
      },
      {
        path: '/announcement',
        component: () => import('@/views/social/announcement/add'),
        name: 'announcement-add',
        hidden: true,
        meta: { title: '添加公告', icon: 'table' }
      }
    ]
  },

  { path: '*', redirect: '/404', hidden: true }
]

export default new Router({
  // mode: 'history', //后端支持可开
  scrollBehavior: () => ({ y: 0 }),
  routes: constantRouterMap
})
